@@ -68,17 +68,40 @@ public class O365Authenticator implements ExchangeAuthenticator {
 
             // force consent
             //uriBuilder.addParameter("prompt", "consent");
-            // switch to new v2.0 OIDC compliant endpoint https://docs.microsoft.com/en-us/azure/active-directory/develop/azure-ad-endpoint-comparison
-            if (Settings.getBooleanProperty("davmail.enableOidc", false)) {
-                uriBuilder.setPath("/" + tenantId + "/oauth2/v2.0/authorize")
-                        .addParameter("scope", "openid " + Settings.getOutlookUrl() + "/EWS.AccessAsUser.All");
+
+            if ("https://outlook.live.com".equals(Settings.getOutlookUrl())) {
+                // live.com endpoint, able to obtain a token but EWS endpoint does not work
+                String liveAuthorizeUrl = "https://login.live.com/oauth20_authorize.srf";
+                uriBuilder = new URIBuilder(liveAuthorizeUrl)
+                        .addParameter("client_id", clientId)
+                        .addParameter("response_type", "code")
+                        .addParameter("redirect_uri", redirectUri)
+                        .addParameter("response_mode", "query")
+                        .addParameter("login_hint", username)
+                        .addParameter("scope", "openid offline_access https://outlook.live.com/EWS.AccessAsUser.All Mail.ReadWrite MailboxSettings.Read")
+                        .addParameter("resource", "https://outlook.live.com")
+                //.addParameter("prompt", "consent")
+                ;
+
             } else if (Settings.getBooleanProperty("davmail.enableGraph", false)) {
-                uriBuilder.setPath("/" + tenantId + "/oauth2/authorize")
-                        .addParameter("resource", "https://graph.microsoft.com");
-                // OIDC compliant
-                //uriBuilder.setPath("/" + tenantId + "/oauth2/v2.0/authorize")
-                //        .addParameter("scope", "Mail.ReadWrite Calendars.ReadWrite MailboxSettings.Read");
-                        //.addParameter("scope", "openid " + Settings.getOutlookUrl() + "/EWS.AccessAsUser.All AuditLog.Read.All Calendar.ReadWrite Calendars.Read.Shared Calendars.ReadWrite Contacts.ReadWrite DataLossPreventionPolicy.Evaluate Directory.AccessAsUser.All Directory.Read.All Files.Read Files.Read.All Files.ReadWrite.All Group.Read.All Group.ReadWrite.All InformationProtectionPolicy.Read Mail.ReadWrite Mail.Send Notes.Create Organization.Read.All People.Read People.Read.All Printer.Read.All PrintJob.ReadWriteBasic SensitiveInfoType.Detect SensitiveInfoType.Read.All SensitivityLabel.Evaluate Tasks.ReadWrite TeamMember.ReadWrite.All TeamsTab.ReadWriteForChat User.Read.All User.ReadBasic.All User.ReadWrite Users.Read");
+                if (Settings.getBooleanProperty("davmail.enableOidc", false)) {
+                    // OIDC compliant
+                    uriBuilder.setPath("/" + tenantId + "/oauth2/v2.0/authorize")
+                            .addParameter("scope", Settings.getProperty("davmail.oauth.scope", "openid profile offline_access Mail.ReadWrite Calendars.ReadWrite MailboxSettings.Read Mail.ReadWrite.Shared Contacts.ReadWrite Tasks.ReadWrite Mail.Send"));
+                    // return scopes with 00000003-0000-0000-c000-000000000000 scopes
+                    //.addParameter("scope", "openid profile offline_access .default");
+                    //.addParameter("scope", "openid profile offline_access "+Settings.getGraphUrl()+"/.default");
+                    //.addParameter("scope", "openid " + Settings.getOutlookUrl() + "/EWS.AccessAsUser.All AuditLog.Read.All Calendar.ReadWrite Calendars.Read.Shared Calendars.ReadWrite Contacts.ReadWrite DataLossPreventionPolicy.Evaluate Directory.AccessAsUser.All Directory.Read.All Files.Read Files.Read.All Files.ReadWrite.All Group.Read.All Group.ReadWrite.All InformationProtectionPolicy.Read Mail.ReadWrite Mail.Send Notes.Create Organization.Read.All People.Read People.Read.All Printer.Read.All PrintJob.ReadWriteBasic SensitiveInfoType.Detect SensitiveInfoType.Read.All SensitivityLabel.Evaluate Tasks.ReadWrite TeamMember.ReadWrite.All TeamsTab.ReadWriteForChat User.Read.All User.ReadBasic.All User.ReadWrite Users.Read");
+                } else {
+                    // Outlook desktop relies on classic authorize endpoint
+                    // on graph endpoint default scopes are scope -> AuditLog.Create Calendar.ReadWrite Calendars.Read.Shared Calendars.ReadWrite Contacts.ReadWrite DataLossPreventionPolicy.Evaluate Directory.AccessAsUser.All Directory.Read.All Files.Read Files.Read.All Files.ReadWrite.All FileStorageContainer.Selected Group.Read.All Group.ReadWrite.All InformationProtectionPolicy.Read Mail.ReadWrite Mail.Send Notes.Create Organization.Read.All People.Read People.Read.All Printer.Read.All PrinterShare.ReadBasic.All PrintJob.Create PrintJob.ReadWriteBasic Reports.Read.All SensitiveInfoType.Detect SensitiveInfoType.Read.All SensitivityLabel.Evaluate Tasks.ReadWrite TeamMember.ReadWrite.All TeamsTab.ReadWriteForChat User.Read.All User.ReadBasic.All User.ReadWrite Users.Read
+                    uriBuilder.setPath("/" + tenantId + "/oauth2/authorize")
+                            .addParameter("resource", Settings.getGraphUrl());
+                }
+                // Probably irrelevant except for graph api, see above, switch to new v2.0 OIDC compliant endpoint https://docs.microsoft.com/en-us/azure/active-directory/develop/azure-ad-endpoint-comparison
+            } else if (Settings.getBooleanProperty("davmail.enableOidc", false)) {
+                uriBuilder.setPath("/" + tenantId + "/oauth2/v2.0/authorize")
+                        .addParameter("scope", Settings.getProperty("davmail.oauth.scope", "openid profile offline_access " + Settings.getOutlookUrl() + "/EWS.AccessAsUser.All"));
             } else {
                 uriBuilder.setPath("/" + tenantId + "/oauth2/authorize")
                         .addParameter("resource", Settings.getOutlookUrl());
@@ -127,7 +150,7 @@ public class O365Authenticator implements ExchangeAuthenticator {
         // common DavMail client id
         String clientId = Settings.getProperty("davmail.oauth.clientId", "facd6cff-a294-4415-b59f-c5b01937d7bd");
         // standard native app redirectUri
-        String redirectUri = Settings.getProperty("davmail.oauth.redirectUri", Settings.getO365LoginUrl()+"/common/oauth2/nativeclient");
+        String redirectUri = Settings.getProperty("davmail.oauth.redirectUri", Settings.getO365LoginUrl() + "/common/oauth2/nativeclient");
         // company tenantId or common
         tenantId = Settings.getProperty("davmail.oauth.tenantId", "common");
 
@@ -146,12 +169,20 @@ public class O365Authenticator implements ExchangeAuthenticator {
             GetRequest getRequest = new GetRequest(url);
             String responseBodyAsString = executeFollowRedirect(httpClientAdapter, getRequest);
             String code;
-            if (!responseBodyAsString.contains("Config=")) {
+            if (!responseBodyAsString.contains("Config=") && responseBodyAsString.contains("ServerData =")) {
+                // live.com form
+                JSONObject config = extractServerData(responseBodyAsString);
+
+                String referer = getRequest.getURI().toString();
+                code = authenticateLive(httpClientAdapter, config, referer);
+            } else if (!responseBodyAsString.contains("Config=")) {
                 // we are no longer on Microsoft, try ADFS
                 code = authenticateADFS(httpClientAdapter, responseBodyAsString, url);
             } else {
                 JSONObject config = extractConfig(responseBodyAsString);
 
+                checkConfigErrors(config);
+
                 String context = config.getString("sCtx"); // csts request
                 String apiCanary = config.getString("apiCanary"); // canary for API calls
                 String clientRequestId = config.getString("correlationId");
@@ -163,7 +194,7 @@ public class O365Authenticator implements ExchangeAuthenticator {
 
                 String referer = getRequest.getURI().toString();
 
-                RestRequest getCredentialMethod = new RestRequest(Settings.getO365LoginUrl()  + "/" + tenantId + "/GetCredentialType");
+                RestRequest getCredentialMethod = new RestRequest(Settings.getO365LoginUrl() + "/" + tenantId + "/GetCredentialType");
                 getCredentialMethod.setRequestHeader("Accept", "application/json");
                 getCredentialMethod.setRequestHeader("canary", apiCanary);
                 getCredentialMethod.setRequestHeader("client-request-id", clientRequestId);
@@ -195,7 +226,7 @@ public class O365Authenticator implements ExchangeAuthenticator {
                     LOGGER.debug("Detected ADFS, redirecting to " + federationRedirectUrl);
                     code = authenticateRedirectADFS(httpClientAdapter, federationRedirectUrl, url);
                 } else {
-                    PostRequest logonMethod = new PostRequest(Settings.getO365LoginUrl()  + "/" + tenantId + "/login");
+                    PostRequest logonMethod = new PostRequest(Settings.getO365LoginUrl() + "/" + tenantId + "/login");
                     logonMethod.setRequestHeader("Accept", "text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8");
                     logonMethod.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
 
@@ -213,7 +244,8 @@ public class O365Authenticator implements ExchangeAuthenticator {
                     URI location = logonMethod.getRedirectLocation();
 
                     if (responseBodyAsString != null && responseBodyAsString.contains("arrUserProofs")) {
-                        location = handleMfa(httpClientAdapter, logonMethod, username, clientRequestId);
+                        logonMethod = handleMfa(httpClientAdapter, logonMethod, username, clientRequestId);
+                        location = logonMethod.getRedirectLocation();
                     }
 
                     if (location == null || !location.toString().startsWith(redirectUri)) {
@@ -223,11 +255,15 @@ public class O365Authenticator implements ExchangeAuthenticator {
                             LOGGER.warn("Authentication successful but user consent or validation needed, please open the following url in a browser");
                             LOGGER.warn(url);
                             throw new DavMailAuthenticationException("EXCEPTION_AUTHENTICATION_FAILED");
+                        } else if ("ConvergedChangePassword".equals(config.optString("pgid"))) {
+                            throw new DavMailAuthenticationException("EXCEPTION_AUTHENTICATION_FAILED_PASSWORD_EXPIRED");
                         } else if ("50126".equals(config.optString("sErrorCode"))) {
                             throw new DavMailAuthenticationException("EXCEPTION_AUTHENTICATION_FAILED");
                         } else if ("50125".equals(config.optString("sErrorCode"))) {
                             throw new DavMailAuthenticationException("LOG_MESSAGE", "Your organization needs more information to keep your account secure, authenticate once in a web browser and try again");
-                        } else if (config.optString("strServiceExceptionMessage") != null) {
+                        } else if ("50128".equals(config.optString("sErrorCode"))) {
+                            throw new DavMailAuthenticationException("LOG_MESSAGE", "Invalid domain name - No tenant-identifying information found in either the request or implied by any provided credentials.");
+                        } else if (config.optString("strServiceExceptionMessage", null) != null) {
                             LOGGER.debug("O365 returned error: " + config.optString("strServiceExceptionMessage"));
                             throw new DavMailAuthenticationException("EXCEPTION_AUTHENTICATION_FAILED");
                         } else {
@@ -257,6 +293,48 @@ public class O365Authenticator implements ExchangeAuthenticator {
 
     }
 
+    private void checkConfigErrors(JSONObject config) throws DavMailAuthenticationException {
+        if (config.optString("strServiceExceptionMessage", null) != null) {
+            throw new DavMailAuthenticationException("EXCEPTION_AUTHENTICATION_FAILED_REASON", config.optString("strServiceExceptionMessage"));
+        }
+    }
+
+    private String authenticateLive(HttpClientAdapter httpClientAdapter, JSONObject config, String referer) throws JSONException, IOException {
+        String urlPost = config.getString("urlPost");
+        PostRequest logonMethod = new PostRequest(urlPost);
+        logonMethod.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
+        logonMethod.setRequestHeader("Referer", referer);
+        String sFTTag = config.optString("sFTTag");
+        String ppft = "";
+        if (sFTTag.contains("value=")) {
+            ppft = sFTTag.substring(sFTTag.indexOf("value=\"")+7, sFTTag.indexOf("\"/>"));
+        }
+
+        logonMethod.setParameter("PPFT", ppft);
+
+        logonMethod.setParameter("login", username);
+        logonMethod.setParameter("loginfmt", username);
+
+        logonMethod.setParameter("passwd", password);
+
+        String responseBodyAsString = httpClientAdapter.executePostRequest(logonMethod);
+        URI location = logonMethod.getRedirectLocation();
+        if (location == null) {
+            if (responseBodyAsString.contains("ServerData =")) {
+                String errorMessage = extractServerData(responseBodyAsString).optString("sErrTxt");
+                throw new IOException("Live.com authentication failure: "+errorMessage);
+            }
+        } else {
+            String query = location.getQuery();
+            if (query.contains("code=")) {
+                String code = query.substring(query.indexOf("code=") + 5);
+                LOGGER.debug("Authentication Code: " + code);
+                return code;
+            }
+        }
+        throw new IOException("Unknown Live.com authentication failure");
+    }
+
     private String authenticateRedirectADFS(HttpClientAdapter httpClientAdapter, String federationRedirectUrl, String authorizeUrl) throws IOException, JSONException {
         // get ADFS login form
         GetRequest logonFormMethod = new GetRequest(federationRedirectUrl);
@@ -371,7 +449,8 @@ public class O365Authenticator implements ExchangeAuthenticator {
 
             // MFA triggered after device authentication
             if (result == null && responseBodyAsString != null && responseBodyAsString.contains("arrUserProofs")) {
-                result = handleMfa(httpClient, processMethod, username, null);
+                processMethod = handleMfa(httpClient, processMethod, username, null);
+                result = processMethod.getRedirectLocation();
             }
 
             if (result == null) {
@@ -382,14 +461,14 @@ public class O365Authenticator implements ExchangeAuthenticator {
         return result;
     }
 
-    private URI handleMfa(HttpClientAdapter httpClientAdapter, PostRequest logonMethod, String username, String clientRequestId) throws IOException, JSONException {
+    private PostRequest handleMfa(HttpClientAdapter httpClientAdapter, PostRequest logonMethod, String username, String clientRequestId) throws IOException, JSONException {
         JSONObject config = extractConfig(logonMethod.getResponseBodyAsString());
         LOGGER.debug("Config=" + config);
 
         String urlBeginAuth = config.getString("urlBeginAuth");
         String urlEndAuth = config.getString("urlEndAuth");
         // Get processAuth url from config
-        String urlProcessAuth = config.optString("urlPost", Settings.getO365LoginUrl()  + "/" + tenantId + "/SAS/ProcessAuth");
+        String urlProcessAuth = config.optString("urlPost", Settings.getO365LoginUrl() + "/" + tenantId + "/SAS/ProcessAuth");
 
         boolean isMFAMethodSupported = false;
         String chosenAuthMethodId = null;
@@ -538,7 +617,7 @@ public class O365Authenticator implements ExchangeAuthenticator {
             throw new IOException("Authentication failed: " + config);
         }
 
-        String authMethod = "PhoneAppOTP";
+        String authMethod = chosenAuthMethodId;
         String type = "22";
 
         context = config.getString("Ctx");
@@ -556,7 +635,7 @@ public class O365Authenticator implements ExchangeAuthenticator {
         processAuthMethod.setParameter("hpgrequestid", hpgrequestid);
 
         httpClientAdapter.executePostRequest(processAuthMethod);
-        return processAuthMethod.getRedirectLocation();
+        return processAuthMethod;
 
     }
 
@@ -580,7 +659,8 @@ public class O365Authenticator implements ExchangeAuthenticator {
     private String executeFollowRedirect(HttpClientAdapter httpClientAdapter, GetRequest getRequest) throws IOException {
         LOGGER.debug(getRequest.getURI());
         ResponseWrapper responseWrapper = httpClientAdapter.executeFollowRedirect(getRequest);
-        if (responseWrapper.getURI().getHost().endsWith("okta.com")) {
+        String responseHost = responseWrapper.getURI().getHost();
+        if (responseHost.endsWith("okta.com")) {
             throw new DavMailAuthenticationException("LOG_MESSAGE", "Okta authentication not supported, please try O365Interactive");
         }
         return responseWrapper.getResponseBodyAsString();
@@ -595,6 +675,21 @@ public class O365Authenticator implements ExchangeAuthenticator {
         }
     }
 
+    /**
+     * Live.com logon form information
+     * @param content response form
+     * @return parsed configuration json
+     * @throws IOException on error
+     */
+    public JSONObject extractServerData(String content) throws IOException {
+        try {
+            return new JSONObject(extract("ServerData =([^\n]+);", content));
+        } catch (JSONException e1) {
+            LOGGER.debug(content);
+            throw new IOException("Unable to extract config from response body");
+        }
+    }
+
     public String extract(String pattern, String content) throws IOException {
         String value;
         Matcher matcher = Pattern.compile(pattern).matcher(content);
